#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>

#include "../include/tdsio.h"
#include "../include/tdssdh.h"
#include "examples.h"


static void usage(void);
static const char *progName;

static void
usage()
{
	fprintf(stderr,
	    "usage: %s [-av] [-i file] syncpattern\n",
	    progName);
	exit(EXIT_FAILURE);
}

int
main(int argc, char **argv)
{
	FILE *inFp;
	struct tdssdh sdhio;
	struct tdsaux auxio;
	size_t nread, skip0, skip1, i, count;
	unsigned int curr_time, last_time;
	int syncpattern, auxflag, verbose, opt;
	unsigned char data;
	char *infile;
	extern int optind;
	extern char *optarg;
long hoge = 0;

	inFp = stdin;
	skip0 = skip1 = 0;
	auxflag = 0;
	verbose = 0;
	infile = NULL;

	progName = argv[0];
	while((opt = getopt(argc, argv, "ai:v")) != EOF) {
		switch(opt) {
		case 'a':
			auxflag = 1;
			break;
		case 'i':	/* input file */
			infile = optarg;
			break;
		case 'v':
			verbose = 1;
			break;
		case '?':
		default:
			usage();
			/* NOTREACHED */
		}
	}
	argc -= optind;
	argv += optind;
	if (argc != 1)
		usage();
	syncpattern = (int)strtoul(argv[0], NULL, 0);

	if (infile == NULL) {
		infile = "(stdin)";
	} else if ((inFp = fopen(infile, "r")) == NULL) {
		fprintf(stderr, "%s: %s: %s\n",
			progName, infile, strerror(errno));
		return -1;
	}
		
	for(i = 0; (nread = fread(&sdhio, sizeof(struct tdssdh), 1, inFp)) == 1;
	    i++) {
		if (sdhio.syncptn0 != 0xffff || sdhio.syncptn1 != 0xffff ||
		    sdhio.syncptn2 != syncpattern) {
			fprintf(stderr, ": %d sync pattern not found\n", i);
			break;
		}
		if (skip0 != 0)
			if ((nread = fread(&auxio, skip0, 1, inFp)) != 1) {
				fprintf(stderr, ": %d aux read error\n", i);
				break;
			}
		if (skip1 == 0) {
			unsigned int rate  = value2key(sdhio.fsample, Freq);
			unsigned int width = value2key(sdhio.bitwidth, Bitw);
			unsigned int chann = value2key(sdhio.chmode, Chan);

			skip1  = rate * width * chann / 8 /*bit*/;
			last_time =
			    (sdhio.totalsec0 | sdhio.totalsec1 << 16) - 1;

			if (auxflag) {
				char auxbuf[256];

				if ((nread = fread(
				    &auxio, sizeof(uint16_t), 1, inFp)) != 1) {
					fprintf(stderr,
					    ": %d aux header read error\n", i);
					break;
				}
				if ((nread = fread(
				    auxbuf, auxio.size, 1, inFp)) != 1) {
					fprintf(stderr,
					    ": %d aux data read error\n", i);
					break;
				}
				skip0 = sizeof(uint16_t) + auxio.size;
			}

			if (verbose) {
				printf("sampling: %dHz/%dbit/%dCH\n",
				    rate, width, chann);
				printf("bytes/1pps: %d\n", skip1);
				printf("start-time: %6d\n", last_time + 1);

				if (auxflag) {
					printf("AUX Ver %d.%d\n",
					    auxio.major, auxio.minor);
					printf("AUX Field Size %d\n",
					    auxio.size);
				}
			}
		}
		curr_time = sdhio.totalsec0 | sdhio.totalsec1 << 16;
		if (curr_time != last_time + 1) {
			fprintf(stderr, ": %d uncontig time-of-day value\n", i);
			break;
		}
		last_time = curr_time;
		if (verbose) {
			printf(".");
			if ((i % 50) == 49)
				printf(": %d\n", i+1);
			fflush(stdout);
		}
		for(count = skip1; count > 0; count--) {
			if ((nread = fread(&data, sizeof(data), 1, inFp)) != 1)
				break;
		}
	}
	if (nread < 0) {
	    fprintf(stderr, "%s: %s: %s\n", progName, infile, strerror(errno));
	    return -1;
	}
	printf("\n");
	return 0;
}
